# Generated by roxygen2: do not edit by hand

export(Bi)
export(DisconnectivityGraph)
export(ELA)
export(ELPruning)
export(Energy)
export(Findbp)
export(Formatting)
export(GELAObj)
export(GradELA)
export(GradientDescent)
export(HeatBath)
export(MinTippingPath)
export(OnestepHBS)
export(PCplot)
export(SSentropy)
export(Stability)
export(SteepestDescent)
export(bin2id)
export(bootstrap_ELA)
export(bootstrap_SA)
export(deltaE_histogram)
export(gStability)
export(hb.paramgen)
export(id2bin)
export(plotSAtest)
export(runSA)
export(sa2params)
export(saEndpoint)
export(showDG)
export(showGELA3D)
export(showIntrGraph)
export(showSSD)
export(showSSD_ggplot)
export(sstable)
export(tptable)
import(foreach)
importFrom(RColorBrewer,brewer.pal)
importFrom(Rcpp,sourceCpp)
importFrom(data.table,transpose)
importFrom(doParallel,registerDoParallel)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_map)
importFrom(dplyr,left_join)
importFrom(dplyr,select)
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_path)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_color_viridis_c)
importFrom(ggplot2,scale_size)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,xlab)
importFrom(graphics,lines)
importFrom(igraph,components)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(magrittr,"%>%")
importFrom(parallel,clusterCall)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(purrr,map)
importFrom(rlang,.data)
importFrom(stats,embed)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,smooth.spline)
importFrom(stringdist,stringdist)
importFrom(stringr,str_c)
importFrom(tidyr,all_of)
importFrom(tidyr,gather)
importFrom(tidyr,pivot_longer)
importFrom(utils,combn)
importFrom(utils,head)
useDynLib(ELAplus, .registration=TRUE)
