# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

SA <- function(ocData, we = 0.001, totalit = 1000L, runadamW = TRUE, sparse = TRUE, lambda = 0.01, intv = 100L, maxlearningrate = 0.005, warmup_tt = 1) {
    .Call(`_ELAplus_SA`, ocData, we, totalit, runadamW, sparse, lambda, intv, maxlearningrate, warmup_tt)
}

fullSA <- function(ocData, envData, runadamW = TRUE, sparse = TRUE, we = 0.001, totalit = 1000L, lambda = 0.01, intv = 100L, maxlearningrate = 0.005, warmup_tt = 1) {
    .Call(`_ELAplus_fullSA`, ocData, envData, runadamW, sparse, we, totalit, lambda, intv, maxlearningrate, warmup_tt)
}

cEnergy <- function(state, alpha, beta) {
    .Call(`_ELAplus_cEnergy`, state, alpha, beta)
}

SteepestDescent_cpp <- function(state, alpha, beta) {
    .Call(`_ELAplus_SteepestDescent_cpp`, state, alpha, beta)
}

SSestimate <- function(alpha, beta, itr = 20000L) {
    .Call(`_ELAplus_SSestimate`, alpha, beta, itr)
}

FindingTippingpoint_cpp <- function(s1, s2, alpha, jj, tmax = 10000L) {
    .Call(`_ELAplus_FindingTippingpoint_cpp`, s1, s2, alpha, jj, tmax)
}

TPestimate <- function(comb, minset, alpha, beta, tmax = 10000L) {
    .Call(`_ELAplus_TPestimate`, comb, minset, alpha, beta, tmax)
}

SSentropy_cpp <- function(uoc, ss, alpha, beta, seitr = 1000L, convTime = 10000L) {
    .Call(`_ELAplus_SSentropy_cpp`, uoc, ss, alpha, beta, seitr, convTime)
}

