% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runELA.R
\name{GradELA}
\alias{GradELA}
\title{GradELA}
\usage{
GradELA(
  sa = sa,
  pruning_type = "relative",
  eid = NULL,
  enmat = enmat,
  env = NULL,
  range = NULL,
  steps = 16,
  th = 0.05,
  threads = 1,
  SS.itr = 20000,
  FindingTip.itr = 10000,
  lower_qr = 0.05,
  bs_params = NULL,
  ocmat = NULL
)
}
\arguments{
\item{sa}{output of runSA function.}

\item{pruning_type}{the type of pruning \code{"relative"}(use with \code{th}) or \code{"bootstrap"} (use with \code{lower_qr},\code{ocmat} and \code{bs_params})}

\item{eid}{target environmental factor: the specified environmental factor to be changed.}

\item{enmat}{array/matrix of environmental factors for each sample (normalization required).}

\item{env}{array of environmental factor values. 
For the environmental parameters that are not specified for \code{eid}, these values are used as references.
If not specified, then the mean of values in \code{enmat} are used as references.}

\item{range}{range over which the environmental factor specified by eid is changed.}

\item{steps}{number of steps in the range.}

\item{th}{ratio of the depth of the basin to be pruned with respect to the deepest basin.}

\item{threads}{number of cores (threads) used for calculation.}

\item{SS.itr}{number of iterations to identify stable states.}

\item{FindingTip.itr}{number of iterations to identify tipping points.}

\item{lower_qr}{Numeric value. Quantile used for constructing confidence intervals from bootstrap samples to check 
whether the estimated energy gaps include zero. Default is \code{lower_qr = 0.05},necessary when \code{type="bootstrap"}.}

\item{bs_params}{output of \link{bootstrap_SA}. list of estimated parameters using bootstrap resampled data set.
necessary when \code{type="bootstrap"}.}

\item{ocmat}{community composition matrix.}
}
\value{
A list of three elements: (1) ELA results for each step (pruned),
  (2) the sequence of environmental values, and (3) the reference env vector.
}
\description{
Estimate energy landscape across environmental gradients.

Runs \link{ELA} and \link{ELPruning} comprehensively across specified environmental parameters.
One environmental parameter can be changed by specifying \code{eid} over \code{steps} within \code{range}. 
If \code{env} is not given, the mean vector of \code{enmat} is used as the reference environmental condition.
}
