% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gels.R
\name{GELAObj}
\alias{GELAObj}
\title{GELAObj}
\usage{
GELAObj(gela, sa, threads = 1)
}
\arguments{
\item{gela}{GELA object/list}

\item{sa}{Parameter list generated by \link{runSA}}

\item{threads}{Number of parallel workers.}
}
\value{
A list containing surface grid data and state-specific surface points.
}
\description{
Construct smoothed energy landscape surface objects from GELA output and parameters.
}
