% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{showSSD}
\alias{showSSD}
\title{showSSD: draw stable state diagram}
\usage{
showSSD(gela)
}
\arguments{
\item{gela}{The output of \link{GradELA}. Object containing stable state lists, energies, the factor,
and environment labels as used in the code.}
}
\value{
NULL (produces a plot as side effect).
}
\description{
Plots energy values of stable states across a factor (e.g., environmental or
control parameter) using the output from \code{gela}. Each unique stable state is
drawn as a separate line with points, and a legend is added.
}
