% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{bin2id}
\alias{bin2id}
\title{bin2id: map a binary vector to a 64base characters}
\usage{
bin2id(bin)
}
\arguments{
\item{bin}{A binary vector (0/1, community composition) to encode.}
}
\value{
A character scalar representing the encoded id.
}
\description{
Encode a binary vector into a base-64 id string index.
}
