% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runSA.R
\name{saEndpoint}
\alias{saEndpoint}
\title{Extract endpoint parameters from \link{runSA} output}
\usage{
saEndpoint(fittingMat, ocmat, enmat = NULL)
}
\arguments{
\item{fittingMat}{A list of SA trajectories returned by \code{runSA(getall = TRUE)}.}

\item{ocmat}{Numeric matrix (0/1). Rows are samples and columns are species.}

\item{enmat}{Optional environmental parameter matrix used in fitting.}
}
\value{
A numeric matrix of median endpoint parameters across serials. 
Rownames correspond to samples and Colnames correspond to species list
}
\description{
Summarize the final parameter estimates across SA replicates by averaging
the last recorded block of parameters.
}
