% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_dataset.R
\docType{data}
\name{basemetadata}
\alias{basemetadata}
\title{Environmental metadata for 256 samples}
\format{
A data frame with 256 rows and 2 variables:
}
\source{
Generated for example purposes (replace with actual source if applicable)
}
\usage{
basemetadata
}
\description{
`basemetadata` is a data frame containing environmental parameters
associated with the same 256 samples as in \code{baseabtable}.
}
\details{
Each row corresponds to a sample, and each column represents
an environmental variable.


The rows in this data frame correspond to the same samples and
ordering as in \code{baseabtable}. These variables can be used
for ELA with environmental parameters such as \code{gradELA}
}
\examples{
data(baseabtable)
data(basemetadata)
head(basemetadata)
formatted <- Formatting(baseabtable = baseabtable, basemetadata = basemetadata)
}
\keyword{datasets}
