% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gels.R
\name{MinTippingPath}
\alias{MinTippingPath}
\title{MinTippingPath}
\usage{
MinTippingPath(s1, s2, hg, j, tmax)
}
\arguments{
\item{s1}{Another integer state ID}

\item{s2}{Another integer state ID}

\item{hg}{explicit/implicit environmental factors}

\item{j}{species interaction matrix}

\item{tmax}{Iteration limit.}
}
\value{
Numeric vector of energies along the selected path (or scalar if s1==s2).
}
\description{
Compute a minimum “tipping” energy path between two states.
}
