% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{showSSD_ggplot}
\alias{showSSD_ggplot}
\title{showSSD_ggplot: draw stable state diagram using ggplot}
\usage{
showSSD_ggplot(gela, plot = TRUE, getSSDobj = TRUE)
}
\arguments{
\item{gela}{The output of \link{GradELA}. Object containing stable state lists, energies, the factor,
and environment labels as used in the code.}

\item{plot}{Logical; if TRUE, show SSD plot}

\item{getSSDobj}{Logical; if TRUE, returns data.frame object used for plotting SSD diagram
Users can manually change the graphical configurations.}
}
\value{
A data frame used for plotting SSD diagram.
}
\description{
Plots energy values of stable states across a factor (e.g., environmental or
control parameter) using the output from \code{gela}. Each unique stable state is
drawn as a separate line with points, and a legend is added.
}
