% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runELA.R
\name{ELA}
\alias{ELA}
\title{Energy landscape analysis in parallel mode}
\usage{
ELA(
  sa = sa,
  env = NULL,
  SS.itr = 20000,
  FindingTip.itr = 10000,
  threads = 1,
  reporting = TRUE,
  fork = TRUE
)
}
\arguments{
\item{sa}{output of runSA function.}

\item{env}{vector of environmental factors for specifying a single environmental condition (optional).}

\item{SS.itr}{number of iterations of the process to identify the stable states.}

\item{FindingTip.itr}{number of iterations of the process to identify the tipping points.}

\item{threads}{number of cores (threads) used for calculation.}

\item{reporting}{enable/disable the reporting function.}

\item{fork}{logical; if TRUE and \code{threads > 1}, a cluster is created internally for parallel execution.}
}
\value{
A list returned by \code{elaconvert}: (1) stable state IDs, (2) stable state energies,
  (3) tipping point ID matrix/list, and (4) tipping energies matrix/list.
}
\description{
Identifying stable states and tipping points.

This function estimates stable states by repeated steepest-descent searches
\code{SSestimate} and then checks tipping points between all pairs of stable
states using \code{FindingTippingpoint_cpp}. 
The result is summarized and converted by \code{elaconvert}
into a list of stable state IDs, their energies, tipping point IDs, and
tipping energies.
}
