% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runELA.R
\name{SteepestDescent}
\alias{SteepestDescent}
\title{SteepestDescent}
\usage{
SteepestDescent(state, alpha, beta)
}
\arguments{
\item{state}{Binary vector representing the initial state.}

\item{alpha}{vector of implicit/explicit environmental effects.}

\item{beta}{Matrix of species interactions.}
}
\value{
A numeric vector of the reached binary stable state and its energy.
}
\description{
Perform steepest descent on the energy landscape.

Starting from an initial binary state, the steepest descent algorithm 
will find a local minimum of the energy landscape. 
The computation is delegated to the C++ implementation
\code{SteepestDescent_cpp}.
}
