% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{showGELA3D}
\alias{showGELA3D}
\title{GELA3D: draw 3D plot of energy landscape}
\usage{
showGELA3D(gelsobj, theta = 25, phi = 50)
}
\arguments{
\item{gelsobj}{List with two elements: surface data frame and line data frame.}

\item{theta}{numeric value. plot3D option: Azimuth angle (in degrees) controlling rotation around the z-axis.}

\item{phi}{numeric value. plot3D option: Elevation angle (in degrees) controlling the vertical viewing position.}
}
\value{
NULL (produces a 3D plot as side effect).
}
\description{
Draws a 3D surface of the energy landscape and overlays
line trajectories using \pkg{plot3D} function.
}
