% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{id2bin}
\alias{id2bin}
\title{id2bin: map a 64base characters to a binary vector}
\usage{
id2bin(id, s)
}
\arguments{
\item{id}{A character scalar encoded by \link{bin2id}.}

\item{s}{Integer number of bits to return.}
}
\value{
An integer vector of 0/1 bits of length s.
}
\description{
Decode a base-64 id string index to a binary vector of length s.
}
