% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eval.R
\name{OnestepHBS}
\alias{OnestepHBS}
\title{Perform one heat-bath update step}
\usage{
OnestepHBS(y, logmo)
}
\arguments{
\item{y}{Numeric matrix of size \code{nproc x nn} (typically 0/1), current states.}

\item{logmo}{Numeric matrix of probabilities compatible with \code{y}. Each row
corresponds to a row in \code{y}; entries are used to sample 0/1 updates.}
}
\value{
A numeric matrix with the same dimensions as \code{y}, updated states.
}
\description{
Updates each row (sample) of a binary state matrix by choosing one random
position per row and resampling it according to precomputed heat-bath
probabilities.
}
