% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runELA.R
\name{Energy}
\alias{Energy}
\title{Energy}
\usage{
Energy(state, alpha, beta)
}
\arguments{
\item{state}{Binary vector representing a state (community composition).}

\item{alpha}{vector of implicit/explicit environmental effects on species.}

\item{beta}{Matrix of species interactions.}
}
\value{
A numeric scalar giving the energy of the state.
}
\description{
Calculate energy of a given state.

Computing energy in \code{state} using given parameters,\code{alpha} and \code{beta}
based on the (extended) maximum pairwise entropy model.
}
