% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runELA.R
\name{SSentropy}
\alias{SSentropy}
\title{SSentropy}
\usage{
SSentropy(state, ss, alpha, beta, seitr = 1000, convTime = 10000)
}
\arguments{
\item{state}{Binary vector (or matrix with one row) representing the current state.}

\item{ss}{Matrix of stable states.}

\item{alpha}{Vector of implicit or explicit environmental effects.}

\item{beta}{Matrix of species interactions.}

\item{seitr}{Number of stochastic search iterations.}

\item{convTime}{Maximum number of iterations for each search.}
}
\value{
A numeric matrix with three columns:
  (1) stable state entropy,
  (2) convergence time,
  (3) number of converged searches.
}
\description{
Estimate stable state entropy for a given state.

Evaluating the entropy of transitions among stable states
starting from a given community state. 

The returned value includes the estimated entropy, the convergence time,
and the number of successful convergences within the given iteration limits.
}
