% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runSA.R
\name{plotSAtest}
\alias{plotSAtest}
\title{plotSAtest: plot the results of SA fitting (Findbp output)}
\usage{
plotSAtest(sa_results, ylim = c("auto", "auto"))
}
\arguments{
\item{sa_results}{Output list from \link{Findbp}.}

\item{ylim}{Numeric length-2 vector (ymin, ymax). Default: \code{c("auto","auto")}.}
}
\value{
A \pkg{ggplot2} object.
}
\description{
Plot mean validation scores over time for the results returned by
\link{Findbp}, including \code{95\%} confidence intervals.
}
\details{
This function uses \pkg{ggplot2} and tidyverse-style data manipulation.
Ensure required packages are installed.
}
