% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runSA.R
\name{sa2params}
\alias{sa2params}
\title{Convert SA output to model parameter objects}
\usage{
sa2params(sa, env = NULL)
}
\arguments{
\item{sa}{A list in the form \code{list(param_matrix, enmat)} where
\code{param_matrix} contains estimated parameters.}

\item{env}{Optional numeric vector of environmental values (column names correspond
to environmental factors).Default is NULL.}
}
\value{
A list with elements \code{he} (h), \code{je} (J), \code{ge} (g or NULL),
  and \code{hge} (combined field).
}
\description{
Convert the output of \link{runSA} / \link{saEndpoint} into parameter
objects in (extended) maximum pairwise entropy model:
\code{h}, \code{J}, and optionally \code{g} when environmental parameter
matrix is specified.
}
