% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{showDG}
\alias{showDG}
\title{showDG: draw DisconnectivityGraph}
\usage{
showDG(
  ela,
  oc,
  label = "",
  bootstrap_ene = NULL,
  min_pval = 0.05,
  plot = TRUE,
  scale_labels = c(0.05, 0.1, 0.2, 0.5),
  getGraphObj = FALSE
)
}
\arguments{
\item{ela}{ELA output tuple/list used by \code{GraphObj}.}

\item{oc}{Community composition matrix/data frame (used to infer \code{s = ncol(oc)}).}

\item{label}{Character; plot title.}

\item{bootstrap_ene}{Optional bootstrap energy matrix/data frame for computing statistical significance levels of energy gaps.}

\item{min_pval}{Numeric; minimum p-value shown in the visualization. This is
usually set to the minimum detection level in bootstrap sampling
(e.g., if the number of bootstrap iterations is 1000, the minimum level is
1/1000 = 0.001).}

\item{plot}{Logical; if TRUE, show disconnectivity graph}

\item{scale_labels}{Numeric vector of values between 0 and 1; labels for the
p-value scale. These values are converted to log-scaled values for plotting.}

\item{getGraphObj}{Logical; if TRUE, returns data.frame object used for plotting DisconnectivityGraph
Users can manually change the graphical configurations.}
}
\value{
A data frame containing energy values, xpositions, pvalues for energy gaps in identified stable states and tipping points.
}
\description{
Computing graph objects from \code{ela} (and optionally
bootstrap energies) and draw the disconnectivity graph for a given community
composition matrix \code{oc}. More than two stable state are required.
}
