% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runELA.R
\name{sstable}
\alias{sstable}
\title{sstable}
\usage{
sstable(ela, ocmat)
}
\arguments{
\item{ela}{output of ELA or the first return value of ELPruning.}

\item{ocmat}{community composition matrix.}
}
\value{
A data frame with columns \code{ID}, \code{Energy}.
}
\description{
Summary of stable states.

Builds a table of stable states and their energies from an ELA output.
The returned data frame contains stable state IDs and the corresponding binary
presence/absence vector (per species). This function is typically used for
inspecting stable states after pruning (if \code{ela} is pruned) or before pruning.
}
